# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"REGION": "app.ipynb",
         "IDENTITY_POOL_ID": "app.ipynb",
         "USER_POOL_ID": "app.ipynb",
         "ACCOUNT_ID": "app.ipynb",
         "CLIENT_ID": "app.ipynb",
         "CLIENT_SECRET": "app.ipynb",
         "COMPULSARY_INPUTS": "app.ipynb",
         "cognito": "app.ipynb",
         "CreateCashier": "app.ipynb",
         "createCashier": "app.ipynb",
         "Auth": "app.ipynb",
         "auth": "app.ipynb",
         "Confirm": "app.ipynb",
         "confirm": "cognito.ipynb",
         "GetProfile": "app.ipynb",
         "getProfile": "app.ipynb",
         "UpdateProfile": "app.ipynb",
         "updateProfile": "app.ipynb",
         "SetPassword": "app.ipynb",
         "setPassword": "app.ipynb",
         "unauth": "app.ipynb",
         "DeleteUser": "app.ipynb",
         "deleteUser": "cognito.ipynb",
         "refresh": "app.ipynb",
         "Cognito": "cognito.ipynb",
         "addUser": "cognito.ipynb",
         "credenFromIdToken": "cognito.ipynb",
         "login": "cognito.ipynb",
         "unAuthCreden": "cognito.ipynb",
         "getAttribute": "cognito.ipynb",
         "loginWithFb": "cognito.ipynb",
         "addVillaEmployee": "cognito.ipynb",
         "update": "cognito.ipynb",
         "refreshToken": "cognito.ipynb",
         "getSecretHash": "cognito.ipynb",
         "disableUser": "cognito.ipynb",
         "enableUser": "cognito.ipynb",
         "setNewPassword": "cognito.ipynb",
         "updateAttribute": "cognito.ipynb"}

modules = ["app.py",
           "cognito.py"]

doc_url = "https://thanakijwanavit.github.io/pynamoLogger/"

git_url = "https://github.com/thanakijwanavit/pynamologger/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: app.ipynb (unless otherwise specified).

__all__ = ['REGION', 'IDENTITY_POOL_ID', 'USER_POOL_ID', 'ACCOUNT_ID', 'CLIENT_ID', 'CLIENT_SECRET',
           'COMPULSARY_INPUTS', 'cognito', 'CreateCashier', 'createCashier', 'Auth', 'auth', 'Confirm', 'confirm',
           'GetProfile', 'getProfile', 'UpdateProfile', 'updateProfile', 'SetPassword', 'setPassword', 'unauth',
           'DeleteUser', 'deleteUser', 'refresh']

# Cell
from awsSchema.apigateway import Response, Event
from .cognito import Cognito
from nicHelper.dictUtil import printDict, filterDt
from nicHelper.exception import  errorString
from datetime import datetime
from dataclasses import dataclass
from dataclasses_json import dataclass_json
from typing import Optional
import ujson as json
import os, traceback, base64, pickle

# Cell
REGION = os.environ.get("REGION")
IDENTITY_POOL_ID = os.environ.get('IDENTITY_POOL_ID')
USER_POOL_ID = os.environ.get('USER_POOL_ID')
ACCOUNT_ID = os.environ.get ('ACCOUNT_ID')
CLIENT_ID = os.environ.get ('CLIENT_ID')
CLIENT_SECRET = os.environ.get('CLIENT_SECRET')
COMPULSARY_INPUTS = json.loads(os.environ.get('COMPULSARY_INPUTS','{}'))


# Cell
cognito = Cognito(
  identityPoolId = IDENTITY_POOL_ID ,
  userPoolId = USER_POOL_ID,
  accountId = ACCOUNT_ID,
  clientId = CLIENT_ID,
  clientSecret = CLIENT_SECRET,
  region = REGION
)

# Cell
@dataclass_json
@dataclass
class CreateCashier:
  user: str
  phone: str
  pw: str
  name: str
  cashierCode: str
  email: Optional[str] = None

# Cell
def createCashier(event, *args):
  try:
    createCashierItem:CreateCashier = Event.parseDataClass(CreateCashier, event)
  except Exception as e:
    return Response.returnError(message = f'{e}')
  ## catch error adding user
  try:
    if not (addResult:=cognito.addUser(**createCashierItem.to_dict()))['success']:
      return Response.returnError(message=addResult)
  except:
    return Response.returnError(message=errorString())
  try:
    updateResult = updateProfile(Event.getInput(
      {'user':createCashierItem.user,
       'attributes':{
         'custom:cashierCode': createCashierItem.cashierCode,
         'email': createCashierItem.email or ''
       }}))
  except:
    return Response.returnError(message=f'update cashier code error {errorString()}')
  return Response.returnSuccess(body={**addResult, **createCashierItem.to_dict(), **updateResult})

# Cell
@dataclass_json
@dataclass
class Auth:
  user: str
  password: str

# Cell
def auth(event, *args):
  ####### validate inputs
  authObject: Auth = Event.parseDataClass(Auth, event)
  print(f'user is {authObject.user} pw is {authObject.password}')
  ####### main function
  try:
    success, token = cognito.login(user = authObject.user, pw = authObject.password)
    if not success: return Response.returnError(message=token)
  except:
    return Response.returnError(message={'error': traceback.format_exc()})
  ##### clean up non-json parsable string ####
  token = filterDt(token)
  return Response.returnSuccess(body=token)

# Cell
@dataclass_json
@dataclass
class Confirm:
  user: str
  code: str

def confirm(event, *args):
  try: confirmItem:Confirm = Event.parseDataClass(Confirm, event)
  except Exception as e: return Response.returnError(f'{e}')

  if not (result := cognito.confirm(**confirmItem.to_dict()))['success']:
    resultString = base64.b64encode(pickle.dumps(result)).decode()
    return Response.returnError(message = {'pickledb64string': resultString, 'error': 'confirmation code error'})
  return Response.returnSuccess(body = result)

# Cell
@dataclass_json
@dataclass
class GetProfile:
  user:str

def getProfile(event, *args):
  ### validate input
  try: inputItem = Event.parseDataClass(GetProfile, event)
  except Exception as e: return Response.returnError(message = f'{e}')
  #### call sdk
  if not (result:= cognito.getAttribute(inputItem.user)):
    return Response.returnError(message = f'{result}')
  return Response.returnSuccess(result)

# Cell
@dataclass_json
@dataclass
class UpdateProfile:
  user: str
  attributes: dict

def updateProfile(event, *args):
  f'''add attributes to the user
    event schema
    body: JSON String
      {UpdateProfile.__doc__}
  '''
  ### validate input
  try: inputItem = Event.parseDataClass(UpdateProfile, event)
  except Exception as e: return Response.returnError(message = f'{e}')
  ### call sdk
  success, result = cognito.updateAttribute(inputItem.user, inputItem.attributes)
  if not success:
    return Response.returnError(message = f'{result}')
  return Response.returnSuccess(body=result)

# Cell
@dataclass_json
@dataclass
class SetPassword:
  user: str
  pw: str

def setPassword(event, *args):
  ### validate input
  try: inputItem = Event.parseDataClass(SetPassword, event)
  except Exception as e: return Response.returnError(message = f'{e}')
  ### call sdk
  success, result = cognito.setNewPassword(inputItem.user, inputItem.pw)
  if not success:
    return Response.returnError(message=f'{result}')
  return Response.returnSuccess(result)

# Cell
def unauth(event, *args):
  unauthCreden = cognito.unAuthCreden()
  cleanedCreden = {k:(v.timestamp() if type(v)==datetime else v) for k,v in unauthCreden.items()}
  return Response.returnSuccess(cleanedCreden)

# Cell
@dataclass_json
@dataclass
class DeleteUser:
  user: str

# Cell
def deleteUser(event,*args):
  body = Event.parseBody(event)
  try: deleteUserObject: DeleteUser = DeleteUser.from_dict(body)
  except: return Response.returnError(message=f'input should have the schema{DeleteUser.__doc__}')
  try: result = cognito.deleteUser(deleteUserObject.user)
  except: return Response.returnError(message=f'{errorString()}')
  return Response.returnSuccess(result)



# Cell
def refresh(event, *args):
  return Response.returnSuccess({})


# AUTOGENERATED! DO NOT EDIT! File to edit: cognito.ipynb (unless otherwise specified).

__all__ = ['Cognito', 'addUser', 'credenFromIdToken', 'login', 'unAuthCreden', 'getAttribute', 'loginWithFb',
           'addVillaEmployee', 'confirm', 'update', 'refreshToken', 'getSecretHash', 'disableUser', 'enableUser',
           'setNewPassword', 'updateAttribute', 'deleteUser']

# Cell
import ujson as json
from pycognito import Cognito as CognitoClient
from pycognito.aws_srp import AWSSRP
from nicHelper.wrappers import add_method
from nicHelper.exception import errorString
from nicHelper.dictUtil import printDict, filterDt
from botocore.exceptions import ClientError

import boto3, botocore, base64, hmac, hashlib


# Cell
class Cognito:
  '''sdk to interact with cognito as a client'''
  def __init__(
    self,
    identityPoolId,
    userPoolId,
    accountId,
    clientId,
    clientSecret,
    region = 'ap-southeast-1'
    ):
    self.identityPoolId = identityPoolId
    self.userPoolId = userPoolId
    self.accountId = accountId
    self.client = boto3.client('cognito-idp', region_name=region)
    self.identityClient = boto3.client('cognito-identity', region_name=region)
    self.clientId = clientId
    self.clientSecret = clientSecret
    self.region = region

  def getCognitoClient(self, user, accessToken = None, idToken = None, refreshToken = None):
    u = CognitoClient(
        self.userPoolId,
        self.clientId,
        client_secret=self.clientSecret,
        username= user,
        access_token = accessToken,
        id_token = idToken
      )
    return u

  def getSrp(self, user, pw):
    return AWSSRP(
        username = user,
        password= pw,
        pool_id = self.userPoolId,
        client_id = self.clientId,
        client_secret= self.clientSecret ,
        client = self.client
      )



# Cell
import botocore
@add_method(Cognito)
def addUser(self,user, phone, pw, name, **kwargs):
  try:
    u = CognitoClient(
      self.userPoolId,
      self.clientId,
      client_secret=self.clientSecret,
      username= user
    )
    u.set_base_attributes(phone_number= phone, name=name )
    u.register(user , pw)
    return {'success': True}
  except boto3.client('cognito-idp').exceptions.UsernameExistsException as e:
    return {'success': False, 'error': 'user exists'}
  except Exception as e:
    return {'success': False, 'error': e , 'traceback': errorString()}


# Cell
@add_method(Cognito)
def credenFromIdToken(self, logins):
  '''get aws credentials from cognito token'''
  ## get identityId
  identityResponse = self.identityClient.get_id(
      AccountId=self.accountId,
      IdentityPoolId=self.identityPoolId,
      Logins = logins
    )
  identityId = identityResponse['IdentityId']
  ## get aws credentials
  credentials_response = self.identityClient.get_credentials_for_identity(
      IdentityId=identityId,
      Logins = logins
      )
  return credentials_response['Credentials']

# Cell
@add_method(Cognito)
def login(self, user:str , pw:str, *args, **kwargs ):
  '''login to pool using cognito'''

  srp = self.getSrp(user,pw)
  try:
    tokens = srp.authenticate_user()
  except ClientError as e:
    print(type(e))
    if e.response['Error']['Code'] == 'NotAuthorizedException':
      return False, 'wrong password'
    return False, 'other problems'

  ##########################################################
  # extract accessToken etc
  accessToken = tokens['AuthenticationResult']['AccessToken']
  refreshToken = tokens['AuthenticationResult']['RefreshToken']
  idToken = tokens['AuthenticationResult']['IdToken']

  ####### Getting the IAM credentials############################
  logins = {f'cognito-idp.ap-southeast-1.amazonaws.com/{self.userPoolId}': idToken}
#   printDict(logins)
  creden = self.credenFromIdToken(logins)
  creden.update(tokens['AuthenticationResult'])
  creden['Expiration'] = creden['Expiration'].timestamp()
  ############ get user attributes
  userResult = self.client.admin_get_user(UserPoolId=self.userPoolId,Username=user)
  userAttributes = userResult.pop('UserAttributes')
  kwUserAttrib = {attrib['Name']:attrib['Value'] for attrib in userAttributes}
#   print(kwUserAttrib)
  creden['userInfo'] = userResult
  creden['kwUserAttrib'] = kwUserAttrib

  return True, filterDt(creden)




# Cell
@add_method(Cognito)
def unAuthCreden(self):
  identityResponse = self.identityClient.get_id(
    AccountId=self.accountId,
    IdentityPoolId=self.identityPoolId
    )
  identityId = identityResponse['IdentityId']
  credentialResponse = self.identityClient.get_credentials_for_identity(
      IdentityId=identityId
      )
  return credentialResponse['Credentials']

# Cell
@add_method(Cognito)
def getAttribute(self, user):
  userResult = self.client.admin_get_user(UserPoolId=self.userPoolId,Username=user)
  try: userResult['UserAttributes'] = {attrib['Name']:attrib['Value'] for attrib in userResult['UserAttributes']}
  except Exception as e: raise Exception(f'error parsing attributes from {userResult}\n{e}')
  return filterDt(userResult)

# Cell
@add_method(Cognito)
def loginWithFb(self, fbKey):
  logins = {'graph.facebook.com':fbKey}
  printDict(logins)
  creden = self.credenFromIdToken(logins)
  creden['Expiration'] = creden['Expiration'].timestamp()
  return creden


# Cell
@add_method(Cognito)
def addVillaEmployee(self,user, cashierCode, name, phone, pw, email = ""):
  try:
    u = CognitoClient(
      self.userPoolId,
      self.clientId,
      client_secret=self.clientSecret,
      username= user
    )
    u.set_base_attributes(phone_number= phone, name=name, address=email)
    u.add_custom_attributes(cashierCode=cashierCode)
    u.register(user , pw)
    return {'success': True}
  except boto3.client('cognito-idp').exceptions.UsernameExistsException as e:
    return {'success': False, 'error': 'user exists'}
  except Exception as e:
    return {'success': False, 'error': e , 'traceback': errorString()}


# Cell
@add_method(Cognito)
def confirm(self, user, code):
  u = CognitoClient(
    self.userPoolId,
    self.clientId,
    client_secret=self.clientSecret,
    username= user
  )
  try:
    result = u.confirm_sign_up(code ,username=user)
    return {'success': True}
  except Exception as e:
    return {'success': False, 'error': e , 'traceback': errorString()}


# Cell
@add_method(Cognito)
def update(user):

  u = Cognito('your-user-pool-id','your-client-id',
    id_token='id-token',refresh_token='refresh-token',
    access_token='access-token')

  u.send_verification(attribute='email')
  return {'update': 'update'}


# Cell
@add_method(Cognito)
def refreshToken(self, user,refreshToken):
    try:
        client = boto3.client('cognito-idp')
#         print(refreshToken)
        return client.initiate_auth(
            ClientId=self.clientId,
            AuthFlow='REFRESH_TOKEN_AUTH',
            AuthParameters={
                'REFRESH_TOKEN': refreshToken,
                'SECRET_HASH': self.getSecretHash(user)
            }
        )
    except botocore.exceptions.ClientError as e:
        return e.response

# Cell
@add_method(Cognito)
def getSecretHash(self, username):
      message = username + self.clientId
      dig = hmac.new(self.clientSecret.encode(), msg=message.encode('UTF-8'),
                     digestmod=hashlib.sha256).digest()
      return base64.b64encode(dig).decode()


# Cell
@add_method(Cognito)
def disableUser(self, username):
  result = self.client.admin_disable_user(
    UserPoolId=self.userPoolId,
    Username=username
  )
  return result

# Cell
@add_method(Cognito)
def enableUser(self, username):
  result = self.client.admin_enable_user(
    UserPoolId=self.userPoolId,
    Username=username
  )
  return result

# Cell
@add_method(Cognito)
def setNewPassword(self, user, newPassword):
  try:
    response = self.client.admin_set_user_password(
      UserPoolId=self.userPoolId,
      Username=user,
      Password=newPassword,
      Permanent=True
    )
  except:
    return False, errorString()
  return True, response

# Cell
@add_method(Cognito)
def updateAttribute(self,user:str, attributes:dict):
  listAttributes = [ {'Name': k, 'Value': v} for k,v in attributes.items()]
  try:
    self.client.admin_update_user_attributes(
      UserPoolId=self.userPoolId,
      Username=user,
      UserAttributes=listAttributes,
    )
    response = self.getAttribute('nic1')
  except Exception as e:
    return False, f'error updating {e}'
  return True, response

# Cell
@add_method(Cognito)
def deleteUser(self,user:str):
  response = self.client.admin_delete_user(
    UserPoolId=self.userPoolId,
    Username=user
  )
  return response